# -------------------------------------------------------
# Licensed Materials - Property of IBM.
# (C) Copyright IBM Corporation 2020
# All Rights Reserved.
#
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by
# GSA ADP Schedule Contract with IBM Corporation.
#
# Purpose : Initialization for Python Mambo module.
#
# -------------------------------------------------------

import mambo

##########################################################
# Extract a string from the inputs given by the user.
# Valid inputs are 0 or 1 command-arguments.
##########################################################
def str_from_cmd(cmd):
    cmd_str = ""
    if (len(cmd)==0):
        cmd_str = ""
    elif (len(cmd)==1):
        if(type(cmd[0]) == str):
            cmd_str = cmd[0]
        else:
            print("Cannot resolve input: ", str(cmd))
    else:
        print("Command takes 0 argument or 1 string argument")

    # print("cmd-str: ", cmd_str)
    return cmd_str

#########################################################################
# Shortcuts for various commands defined in the Python Mambo module
# IN: Tuple with 0 or more entries
# Convert this tuple to a string and pass it to the Mambo module method
#########################################################################
def simdebug(*cmd):
    mambo.simdebug( str_from_cmd(cmd) )
def mysim(*cmd):
    mambo.mysim( str_from_cmd(cmd) )
def quit(*cmd):
    mambo.quit( str_from_cmd(cmd) )
def q(*cmd):
    mambo.q( str_from_cmd(cmd) )
def alias(*cmd):
    mambo.alias( str_from_cmd(cmd) )
def define(*cmd):
    mambo.define( str_from_cmd(cmd) )
def display(*cmd):
    mambo.display( str_from_cmd(cmd) )
def version(*cmd):
    mambo.version( str_from_cmd(cmd) )
def modify(*cmd):
    mambo.modify( str_from_cmd(cmd) )
def object(*cmd):
    mambo.object( str_from_cmd(cmd) )
# Python has its own help() command, hence need a different name
def mambohelp(*cmd):
    mambo.help( str_from_cmd(cmd) )
def mhelp(*cmd):
    mambo.help( str_from_cmd(cmd) )
def helprecursive(*cmd):
    mambo.helprecursive( str_from_cmd(cmd) )
def simstop(*cmd):
    mambo.simstop( str_from_cmd(cmd) )
def trigger(*cmd):
    mambo.trigger( str_from_cmd(cmd) )
def trc(*cmd):
    mambo.trc( str_from_cmd(cmd) )
def uint32_to_float(*cmd):
    mambo.uint32_to_float( str_from_cmd(cmd) )
# Few commands are conditionally defined in the 'mambo' Pythom module.
# Check for the availability of such commands before defining their shortcuts.
if ('mambo_update' in dir(mambo)):
    def mambo_update(*cmd):
        mambo.mambo_update( str_from_cmd(cmd) )
if ('ereader' in dir(mambo)):
    def ereader(*cmd):
        mambo.ereader( str_from_cmd(cmd) )
if ('simemit' in dir(mambo)):
    def simemit(*cmd):
        mambo.simemit( str_from_cmd(cmd) )
# Commands that do not exist in TCL and are defined newly in Python
def lsobjects(*cmd):
    mambo.ls_objects( str_from_cmd(cmd) )
def lssystem(*cmd):
    mambo.ls_system( str_from_cmd(cmd) )
def lsproperties(*cmd):
    mambo.ls_properties( str_from_cmd(cmd) )
def lscores(*cmd):
    mambo.ls_cores( str_from_cmd(cmd) )

##################################################
# Command completion using rlcompleter, readline
##################################################
vars = globals()
vars.update(locals())
try:
    import readline
except ImportError:
    # Silently ignore missing readline module
    pass
else:
    # We don't have to wrap the following import in a 'try', because
    # we already know 'readline' was imported successfully.
    import rlcompleter
    readline.parse_and_bind("tab: complete")
    readline.set_completer(rlcompleter.Completer(vars).complete)

