#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2006, 2017
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
# Utils
#------------------------------------------------------------------------

#source main.tcl

# the following variables save the previous values of the
# registers; to show what happens in a step, we compare the
# values before (in the pXX variable) with the value after
# (in the XX variable), and keep the updated value.

set show_complete_state 1

set setup 0

proc setup_previous { sim } {
    global setup

    set t [$sim cpu 0 display currentthread]

    if { $setup == 0 } {
        foreach spr [$sim cpu $t display spr list] {
            set p($spr) 0
        }
        set setup 1

        for { set i 0 } { $i < 32 } { incr i 1 } {
            eval "global pr$i"
            eval "set pr$i 0"
            eval "global pf$i"
            eval "set pf$i 0"
            eval "global pv$i"
            eval "set pv$i 0"
        }
    }
}


# display the complete state (general purpose registers)
# and remember the value that you displayed.

proc mcm_display_gprs { sim mcm cpu } {
    global p
    set num_fpr [ $sim display nfpr ]
    puts "State of MCM:$mcm CPU:$cpu "

    setup_previous $sim

    for { set i 0 } { $i < 32 } { incr i 1 } {
        eval "global pr$i"
        eval "global pf$i"
        eval "global pv$i"
    }

    set i 0
    foreach spr [$sim mcm $mcm cpu $cpu display spr list] {
        eval "set $spr [$sim mcm $mcm cpu $cpu display spr $spr]"
        set cmd "set p($spr) \$$spr"
        eval $cmd
        puts -nonewline "   $spr=[$sim mcm $mcm cpu $cpu display spr $spr]"
        incr i 1
        if { ( $i % 4 ) == 0 } {
            puts -nonewline "\n"
        }
    }
    puts -nonewline "\n"
    for { set i 0 } { $i < 32 } { incr i 1 } {
        set cmd "set r$i [$sim mcm $mcm cpu $cpu display gpr  $i];   set pr$i \$r$i"
        eval $cmd
        if { $num_fpr != 0 } {
            set cmd "set f$i [$sim mcm $mcm cpu $cpu display fpr  $i];   set pf$i \$f$i"
            eval $cmd
        }
    }


    # Print the GPRS
    for { set i 0 } { $i < 32 } { incr i 1 } {
        if { ( $i % 4 ) == 0 } {
            puts -nonewline "\n    "
        }
        set index [format "%02d" $i]
        set cmd "puts -nonewline \"R$index=\$r$i  \""
        eval $cmd
    }
    puts -nonewline "\n"
    # Print the FPRS
    if { $num_fpr != 0 } {
        for { set i 0 } { $i < 32 } { incr i 1 } {
            if { ( $i % 4 ) == 0 } {
                puts -nonewline "\n    "
            }
            set index [format "%02d" $i]
            set cmd "puts -nonewline \"F$index=\$f$i  \""
            eval $cmd
        }
        puts -nonewline "\n"
    }
}

proc display_gprs { sim } {
    global p
    set num_fpr [ $sim display nfpr ]
    set t [$sim cpu 0 display currentthread]
    puts "State of $t"

    setup_previous $sim

    for { set i 0 } { $i < 32 } { incr i 1 } {
        eval "global pr$i"
        eval "global pf$i"
        eval "global pv$i"
    }

    set i 0
    foreach spr [$sim cpu $t display spr list] {
        eval "set $spr [$sim cpu $t display spr $spr]"
        set cmd "set p($spr) \$$spr"
        eval $cmd
        puts -nonewline "   $spr=[$sim cpu $t display spr $spr]"
        incr i 1
        if { ( $i % 4 ) == 0 } {
            puts -nonewline "\n    "
        }
    }
    puts -nonewline "\n"
    for { set i 0 } { $i < 32 } { incr i 1 } {
        set cmd "set r$i [$sim cpu $t display gpr  $i];   set pr$i \$r$i"
        eval $cmd
        if { $num_fpr != 0 } {
            set cmd "set f$i [$sim cpu $t display fpr  $i];   set pf$i \$f$i"
            eval $cmd
        }
    }


    # Print the GPRS
    for { set i 0 } { $i < 32 } { incr i 1 } {
        if { ( $i % 4 ) == 0 } {
            puts -nonewline "\n    "
        }
        set index [format "%02d" $i]
        set cmd "puts -nonewline \"R$index=\$r$i  \""
        eval $cmd
    }
    puts -nonewline "\n"
    # Print the FPRS
    if { $num_fpr != 0 } {
        for { set i 0 } { $i < 32 } { incr i 1 } {
            if { ( $i % 4 ) == 0 } {
                puts -nonewline "\n    "
            }
            set index [format "%02d" $i]
            set cmd "puts -nonewline \"F$index=\$f$i  \""
            eval $cmd
        }
        puts -nonewline "\n"
    }
}


proc changed_regs { sim } {
    # need previous values
    global show_complete_state
    global p
    set num_fpr [ $sim display nfpr ]

    setup_previous $sim

    for { set i 0 } { $i < 32 } { incr i 1 } {
        eval "global pr$i"
        eval "global pf$i"
        eval "global pv$i"
    }

    if {$show_complete_state == 1} then {
        display_gprs $sim
        set show_complete_state 0
    } else {

        # no change so far
        set change ""
        set t [$sim cpu 0 display currentthread]

        #
        #   Display Changed SPRs
        #
        foreach spr [$sim cpu $t display spr list] {
            eval "set $spr [$sim cpu $t display spr $spr]"
            set cmd "if { \$$spr != \$p($spr) } { append change \" $spr = \$$spr\"; set p($spr) \$$spr }"
            #set cmd "set p($spr) \$$spr"
            eval $cmd

            eval "set $spr [$sim cpu $t display spr $spr]"
            set cmd "if { \$$spr != \$p($spr) } { set changed 1 } else { set changed 0 }"
            eval $cmd
            if { $changed } {
                eval "append change \" $spr = \$$spr\n\""
                set cmd "set p($spr) \$$spr"
                eval $cmd
            }
        }

        #
        #   Display Changed GPRs
        #
        for { set i 0 } { $i < 32 } { incr i 1 } {
            eval "set r$i [$sim cpu $t display gpr $i]"
            set cmd "if { \$r$i != \$pr$i } { set changed 1 } else { set changed 0 }"
            eval $cmd
            if { $changed } {
                eval "append change \" r$i = \$r$i\n\""
                set cmd "set pr$i \$r$i"
                eval $cmd
            }
        }

        #
        #   Display Changed FPRs
        #
        if { $num_fpr != 0 } {
            for { set i 0 } { $i < 32 } { incr i 1 } {
                eval "set f$i [$sim cpu $t display fpr $i]"
                set cmd "if { \$f$i != \$pf$i\ } { set changed 1 } else { set changed 0 }"
                eval $cmd
                if { $changed } {
                    set cmd "append change \" f$i = \$f$i\n\""
                    eval $cmd
                    set cmd "set pf$i \$f$i"
                    eval $cmd
                }
            }
        }

        # now print the sum total of everything that changed
        if { $change != "" } then { puts "$change" }
    }
}

#------------------------------------------------------------------------
# get_current_pc
#     Get the current program counter (IAR) from the target cpu
#------------------------------------------------------------------------
proc get_current_pc { sim cpu } {
    return [ $sim cpu $cpu display spr pc ]
}

#------------------------------------------------------------------------
# get_imem_ra
#     Get the real instruction address given the effective address and cpu
#     Note: itranslate can return an "illegal translation"
#------------------------------------------------------------------------
proc get_imem_ra { sim ea cpu } {
    if { [ catch  { set ra [ $sim cpu $cpu itranslate $ea ] } ] } then {
        return "error"
    } else {
        return $ra
    }
}

#------------------------------------------------------------------------
# get_opcode_from_ra
#     Get the opcode given a real instruction address
#------------------------------------------------------------------------
proc get_opcode_from_ra { sim ra } {
    return [ $sim memory display $ra 4 ]
}

#------------------------------------------------------------------------
# get_opcode_from_ea
#     Get the opcode given an effective instruction address and a cpu
#------------------------------------------------------------------------
proc get_opcode_from_ea { sim ea cpu } {
    set ra [ get_imem_ra $sim $ea $cpu ]
    if { $ra != "error" } then {
        return [ get_opcode_from_ra $sim $ra ]
    } else {
        return "error"
    }
}

#------------------------------------------------------------------------
# get_disasm_from_ra
#     Get the mneumonic given a real instruction address
#------------------------------------------------------------------------
proc get_disasm_from_ra { sim ra } {
    set opcode [ get_opcode_from_ra $sim $ra ]
    if { $opcode == "error" } then {
        return "error"
    } else {
        return [ $sim util ppc_disasm $opcode 0x0 ]
    }
}

#------------------------------------------------------------------------
# get_disasm_from_ea
#     Get the mneumonic given an effective instruction address and a cpu
#------------------------------------------------------------------------
proc get_disasm_from_ea { sim ea cpu } {
    set opcode [ get_opcode_from_ea $sim $ea $cpu ]
    if { $opcode == "error" } then {
        return "error"
    } else {
        return [ $sim util ppc_disasm $opcode 0x0 ]
    }
}

#------------------------------------------------------------------------
# display_pc
#     Here for backwards compatibility with older scripts.
#------------------------------------------------------------------------
proc display_pc { sim {cpu 0} } {

    set pc     [ get_current_pc $sim $cpu ]
    set disasm [ get_disasm_from_ea $sim $pc $cpu ]
    set time   [ $sim display cycles ]
    puts "PC:$pc (Time $time): $disasm"
}


# #############################################################
# always display the complete state if asked to
proc display_state { sim } {
    display_gprs $sim
        display_pc   $sim
}

# or display just what hasn't changed since the last time
# which is the same if nothing has been displayed before
proc changed_state { sim } {
    changed_regs $sim
        display_pc   $sim
}

# #############################################################
# Go until the next call or return
proc cgo { sim } {
    set t [$sim cpu 0 display currentthread]
        # PowerPC return instruction, in hex
    set ret  [expr 0x43800020 + 0]
    ##set hret [format "%08X" $ret]
        # PowerPC call instruction, in hex
        # calls are subject to equality under mask 0xFC000001 (below)
    set call [expr 0x48000001 + 0]
    ##set hcall [format "%08X" $call]

    while { [simstop status] == 0 } {
        $sim cycle 1
        ##changed_state $sim
        set pc [$sim cpu $t display spr pc]
                # itranslate can return an "illegal translation"
                if [catch  { set pc_raddr [$sim util itranslate $pc] } rinst] {
                    set minst     "--------"
                } else {
                    # get current instruction, masked, and in hex
                    set rinst [expr [$sim memory display $pc_raddr 4] + 0]
                    set minst [expr $rinst & 0xFC000001]
                    ##set hinst [format "%08X" $rinst]
                    ##puts "inst is $hinst ($rinst)"
                }
        if { $rinst == $ret } {
            puts "Return reached"
            display_state $sim
            return
        } else {

            ##puts "$hinst != $hret"
        }
        if { $minst == $call } {
            puts "Call reached"
            set foo [exec grep [format "%08x:" $pc] vmlinux.dis]
            puts "Call to $foo"
            display_state $sim
            return
        } else {
            ##puts "$hinst != $hcall"
        }
    }
}

# #############################################################
# a set of procedures that are verbose -- they trace every instruction

# step one instruction
proc vcycle { sim } {
    changed_state $sim
    $sim cycle 1
}
proc vstep { sim } {
   global hooks

   changed_state $sim
   if { [ info exists hooks(vstep_hook) ] } then {
      eval $hooks(vstep_hook)
   }
   $sim step 1
}

# step some specific number of instructions
proc vstepn { sim count } {
    set done 0
    set first_time 1
    while { $done != 1 } {
        # check if we want to stop the simulation
        set stopstat [string trim [simstop status]]
        if { $first_time == 0 &&
             $stopstat != "instruction count," } {
            # something external has stopped the cpu
            set done 1
        } else {
            incr count -1
            vstep $sim
        }
        set first_time 0

        if { $count <= 0 } {
            set done 1
        }
    }
    simstop status
    display_state $sim
}

# go until finished
proc vgo { sim } {
    vcycle $sim
    while { [string compare [string trim [simstop status]] "cycle count"] == 0 } {
        vcycle $sim
    }
}

# #############################################################
# a set of procedures that watch for an address
# wgo -- watch and go
# wvgo -- watch and go verbosely

proc wvgo { sim addr } {
    set t [$sim cpu 0 display currentthread]
    set done 0
    vcycle $sim
    while { $done != 1 } {
        set pc [$sim cpu $t display spr pc]
        # check if we found the address for which we are looking
        # since the two addresses are strings, they may look different,
        # so we subtract them to see if they are equal
        if { [expr $pc-$addr] == 0 } {
            puts "Found address $addr"
            set done 1
        } else {
            # check if we want to stop the simulation
            if { [string compare [string trim [simstop status]] "cycle count"] == 0 } {
                vcycle $sim
            } else {
                # something external has stopped the cpu
                set done 1
            }
        }
    }
}

# #############################################################
# a set of procedures that watch for an address
proc wgo { sim addr } {
    set t [$sim cpu 0 display currentthread]
    set done 0
    $sim cycle 1
    while { $done != 1 } {
        set pc [$sim cpu $t display spr pc]
        # check if we found the address for which we are looking
        # since the two addresses are strings, they may look different,
        # so we subtract them to see if they are equal
        if { [expr $pc-$addr] == 0 } {
            puts "Found address $addr"
            set done 1
        } else {
            # check if we want to stop the simulation
            if { [string compare [string trim [simstop status]] "cycle count"] == 0 } {
                $sim cycle 1
            } else {
                # something external has stopped the cpu
                set done 1
            }
        }
    }
}

# #############################################################
# Running with sampling
#
# run an image from cycles "start" to "end" in steps of "step",
# and at each step print the state of the system.  If you have
# two runs which should be the same, comparing the output of
# the two using sample_segment will narrow down where the first
# change is.
proc sample_trace { sim start step end } {

    # step forward to the suspect part
    $sim cycle $start
    display_state $sim

        set i $start
        while { $i < $end } {
            $sim cycle $step
            display_state $sim
            set i [expr $i+$step]
        }
}

# Cyclesim version of the above
proc sample_trace_cyclesim { sim start step end } {

    # step forward to the suspect part
    $sim step $start
    display_state $sim

        set i $start
        while { $i < $end } {
            $sim step $step
            display_state $sim
            set i [expr $i+$step]
    }
}

#sample_trace mysim 30000000 100000 50000000
#sample_trace mysim 32700000 1000 32800000
#sample_trace mysim 32741000 1 32742000

proc sample_mem {sim start step end } {

    # step forward to the suspect part
    $sim cycle $start

    for { set i $start } { $i < $end } { incr i $step } {
        $sim cycle $step
        set cont [ get_opcode_from_ra $sim 0x3d3371c ]
        set time  [ $sim display cycles ]
        puts "cycle:$time content:$cont"
    }
}

proc sample_inst { sim start step end } {

    # step forward to the suspect part
    $sim cycle $start
    display_pc $sim

    for { set i $start } { $i < $end } { incr i $step } {
        $sim cycle $step
        display_pc $sim
    }
}
# #############################################################
# disassemble a range of memory, from addr to addr+insts
proc disasm_mem { sim addr insts } {
    set index 0
    while { $index < $insts } {
        set eaddr [expr $addr + ($index * 4)]
        set raddr [$sim util itranslate $eaddr]
        set inst [$sim memory display $raddr 4]
        set disasm [$sim util ppc_disasm $inst $eaddr]
        set hex_eaddr [format "0x%08X" $eaddr]
        set hex_raddr [format "0x%08X" $raddr]
        puts "EADDR:$hex_eaddr RADDR:$hex_raddr Enc:$inst : $disasm"

        incr index
    }
}
proc dump_functions { filename } {

    set funcs [object functions $filename]
    foreach func_entry $funcs {
        #puts "func_entry = $func_entry"
        array set func $func_entry
        set func_name($func(FUNC_START)) $func(FUNC)
        #puts "$func(FUNC_START)   $func(FUNC)"
        #foreach ea array names ea_inst
    }
    foreach ea [lsort [array names func_name]] {
        puts "$ea  $func_name($ea)"
    }
}

# #############################################################
# a procedure to dump the page table
# NOTE: this dumps page table for *mysim* not a general sim

proc dump_pagetbl { location length } {

    for { set i $location } { $i < [expr $location + $length] } { incr i 16 } {
        set val1 [mysim memory display $i 8]
        set val2 [mysim memory display [expr $i + 8] 8]]
        if { ( [string compare $val1 0x0] != 0 ) || ( [string compare $val1 0x0] != 0 ) } {
            puts [format "%08X: %s %s" $i [mysim memory display $i 8] [mysim memory display [expr $i + 8] 8]]
        }
    }
}

# #############################################################
# procedures to be called by triggers
# triggers include breakpoints.
# triggers specify a condition and a function.
# they call the function when the condition is true.

proc break_point { sim { reason "breakpoint" } args } {
    simstop
    if { [catch "array set arginfo $args" errs] == 0 } {
        set mcm $arginfo(mcm)
        set cpu $arginfo(cpu)
        set thread $arginfo(thread)
    } else {
        #puts "break_point: failed to parse args with errors $errs"
        #puts "break_point: failed args = $args"
        set mcm 0
        set cpu 0
        set thread 0
    }
    set pc [$sim mcm $mcm cpu $cpu:$thread display spr pc]
    set ra [get_imem_ra $sim $pc $cpu:$thread]
    puts "MCM=$mcm CPU=$cpu:$thread PC=$pc: (real-addr:$ra) break point ($reason)"
    mcm_display_gprs $sim $mcm $cpu:$thread
    simstop
}
#mysim trigger set pc 0x0 "break_point mysim zero"

proc set_break { sim addr reason } {
    eval "$sim trigger set pc $addr \"break_point $sim $reason\""
    eval "$sim trigger set real_pc $addr \"break_point $sim $reason\""
}
proc clear_break { sim addr } {
    eval "$sim trigger clear pc $addr"
    eval "$sim trigger clear real_pc $addr"
}
proc set_cycle_break { sim cycle reason } {
    eval "$sim trigger set cycle $cycle \"break_point $sim $reason\""
}
proc vmlinux_set_break { function_name { sim mysim } } {

    global vmlinux_filename
    
    if { [catch "set entry_info [list [object address $vmlinux_filename $function_name]]" errs] == 0 } {
        if { [catch "array set func_info [list $entry_info]" errs] == 0 } {
            set_break $sim $func_info(FUNC_START) "EA_Kernel_Breakpoint_At_$function_name"
            set_break $sim [expr $func_info(FUNC_START) & 0x3FFFFFFFFFFFFFFF] "PA_Kernel_Breakpoint_At_$function_name"
        } else {
            puts "vmlinux_set_break failed with $errs"
        }
    } else {
        puts "vmlinux_set_break object call failed with $errs"
    }
}
proc vmlinux_set_break_on_return { function_name { sim mysim } } {

    global vmlinux_filename
    
    if { [catch "set entry_info [list [object address $vmlinux_filename $function_name]]" errs] == 0 } {
        if { [catch "array set func_info [list $entry_info]" errs] == 0 } {
            # Now we need to go through all the instructions in function range and set breaks on 0x4E800020 (blr) 
            set inst_data [object inst_data vmlinux $func_info(FUNC_START) $func_info(FUNC_END)]
            array set ea_inst $inst_data
            foreach ea [array names ea_inst] {
                if { $ea_inst($ea) == 0x4e800020 } {
                    set_break $sim $ea "EA_Kernel_Breakpoint_At_$function_name"
                    set_break $sim [expr $ea & 0x3FFFFFFFFFFFFFFF] "PA_Kernel_Breakpoint_At_$function_name"
                }
            }
            set_break $sim $func_info(FUNC_START) "EA_Kernel_Breakpoint_At_$function_name"
            set_break $sim [expr $func_info(FUNC_START) & 0x3FFFFFFFFFFFFFFF] "PA_Kernel_Breakpoint_At_$function_name"
        } else {
            puts "vmlinux_set_break failed with $errs"
        }
    } else {
        puts "vmlinux_set_break object call failed with $errs"
    }
}
proc vmlinux_clear_break { sim function_name } {

    global vmlinux_filename
    
    if { [catch "set entry_info [list [object address $vmlinux_filename $function_name]]" errs] == 0 } {
        if { [catch "array set func_info [list $entry_info]" errs] == 0 } {
            clear_break $sim $func_info(FUNC_START)
            clear_break $sim [expr $func_info(FUNC_START) & 0x3FFFFFFFFFFFFFFF] 
        } else {
            puts "vmlinux_clear_break failed with $errs"
        }
    } else {
        puts "vmlinux_clear_break object call failed with $errs"
    }
}

#mysim trigger set pc 0x0 "break_point mysim zero"

proc dpage_fault { sim args } {
    set t [$sim cpu 0 display currentthread]
    set pc [$sim cpu $t display spr pc]
    set ipc [$sim cpu $t display spr srr0]
    set dsisr [$sim cpu $t display spr dsisr]
    set ea [$sim cpu $t display spr dar]
    puts "$pc: data page fault for PC:$ipc EA:$ea DSISR:$dsisr"
    simstop
}
#mysim trigger set pc 0x300 "dpage_fault mysim"

proc ipage_fault { sim args } {
    set t [$sim cpu 0 display currentthread]
    set pc [$sim cpu $t display spr pc]
    set ipc [$sim cpu $t display spr srr0]
    puts "$pc: inst page fault for PC:$ipc"
    simstop
}
#mysim trigger set pc 0x400 "ipage_fault mysim"

proc illegal_inst { sim args } {
    set t [$sim cpu 0 display currentthread]
    set pc [$sim cpu $t display spr pc]
    set ipc [$sim cpu $t display spr srr0]
    puts "$pc: illegal instruction for PC:$ipc"
    simstop
}
#mysim trigger set pc 0x700 "illegal_inst mysim"

##########################################################
# Bogus disk helpers                                     #
##########################################################

proc cow_disk {sim id file} {
    $sim bogus disk init $id $file newcow ./cow-$id 1024
}

proc rw_disk {sim id file} {
    $sim bogus disk init $id $file rw
}

##########################################################
# Command line options helpers                           #
##########################################################

proc parse_command_line_options { { conf myconf } } {
  # get command line definitions from mambo
  set alldefs [display command_line_config_options]

  foreach def $alldefs {
    if { [lsearch -exact $def appname] != 0 } {
       if { [lsearch -exact $def tcl_proc] == 0 } {
           set tcl_proc_name [lindex $def 1]
           eval "[split $tcl_proc_name /] $conf"
       } else {
           eval "$conf config $def"
       }
    }
  }
}

proc get_user_option_from_command_line { option_name } {
  # get command line definitions from mambo
  set alldefs [display command_line_user_options]

  foreach def $alldefs {
    if { [lsearch -exact $def $option_name] == 0 } {
       return [lrange $def 1 end]
    }
  }

  return ""
}

proc get_user_option_from_command_line_w_default_value { option_name default_value } {
  # get command line definitions from mambo
  set alldefs [display command_line_user_options]

  foreach def $alldefs {
    if { [lsearch -exact $def $option_name] == 0 } {
       return [lrange $def 1 end]
    }
  }

  return $default_value
}

proc get_user_option { option_name default_value } {
     get_user_option_from_command_line_w_default_value $option_name $default_value
}

proc get_application_name_from_command_line { } {
  # get command line definitions from mambo
  set alldefs [display command_line_config_options]

  foreach def $alldefs {
    if { [lsearch -exact $def appname] == 0 } {
       return [lindex $def 1]
    }
  }

  return [get_user_option_from_command_line appname]
}

proc get_config_name_from_command_line { } {
  # get command line definitions from mambo
  set alldefs [display command_line_config_options]

  foreach def $alldefs {
       return [lindex $def 1]
  }

  return ""
}

##########################################################
# Others                                                 #
##########################################################

proc get_string_at_ea {sim ea procid tid} {
    set ra [$sim cpu $procid thread $tid dtranslate $ea]
    set curchar [format "%c" [$sim memory display $ra 1]]
    set retstring ""
    while {[string compare $curchar "\0"]} { #0 is ASCII null string terminator
	set retstring "$retstring$curchar"
        incr ea 1
	set ra [$sim cpu $procid thread $tid dtranslate $ea]
        set curchar [format "%c" [$sim memory display $ra 1]]
    }
    return $retstring
}

proc get_env_bool { variable_name } {
    global env

    if { [llength [array get env $variable_name]] > 0 } {
        if { $env($variable_name) == 0 } {
            return 0;
        } else {
            return 1;
        }
    } else {
        return 0;
    }
}

proc console_input { { sim mysim } { string } } {
  $sim console create input in string "$string"
}

#
# Images Helpers
#

if {![info exists IMAGES]} {
    if {[info exists env(IMAGES_DIR)]} {
        set IMAGES $env(IMAGES_DIR)
    } else {
        set IMAGES ../../../images
    }
}

proc getimage { pathname } {

    global env

    set make "make"
    if {[info exists env(MAKE)]} {
        set make $env(MAKE)
    }


    set separator [string last "/" $pathname]
    set dir [string range $pathname 0 [expr $separator - 1]]
    set target [string range $pathname [expr $separator + 1] end]

    set cwd [pwd]
    cd $env(IMAGES_DIR)
    catch { exec $make -C $env(IMAGES_DIR)/$dir $target } msg
    if { ! [file exists $env(IMAGES_DIR)/$dir/$target] } {
        puts "getimage for $pathname seemed to have failed with msg : $msg"
        return "get-image-failed-for-$pathname"
    }
    cd $cwd
    return "$env(IMAGES_DIR)/$pathname"
}


#
# functions to wait for a console string
#

# We set a console trigger to wait for the desired console string
# The trigger will take us to the following TCL procedure, which
# removes the trigger, and stops the simulation.
#

proc Console_String_Found { sim mcm args } {

    array set triginfo $args

    # get rid of trigger
    $sim mcm $mcm trigger clear console $triginfo(match)
    simstop
}

proc wait_for_console_input { string_to_wait_for { mcm 0 } { sim mysim } } {

    $sim mcm $mcm trigger set console $string_to_wait_for "Console_String_Found $sim $mcm"
    $sim go
}

proc ppc_set_all_sprs { sim spr_name value } {

    set mcms [$sim display number_of_MCMs]
    #set threads [$sim display number_of_threads]
    set cpus [$sim display mcm 0 number_of_cpus]

    for { set m 0 } { $m < $mcms } { incr m } {
        for { set c 0 } { $c < $cpus } { incr c } {
            set threads [$sim mcm $m cpu $c display number_of_threads]
            for { set t 0 } { $t < $threads } { incr t } {
                if { $threads > 1 } {
                    $sim cpu $m:$c:$t set spr $spr_name $value
                } else {
                    $sim cpu $m:$c set spr $spr_name $value
                }
            }
        }
    }
}

proc wait_trigger { sim mcm args } {
    array set triginfo $args
    $sim mcm $mcm trigger clear console $triginfo(match)
    simstop

}

proc wait { str { mcm 0 } { sim mysim } } {

    $sim mcm $mcm trigger set console "$str" "wait_trigger $sim $mcm"
}

proc wait_continue_trigger { str sim mcm args } {
    array set triginfo $args
    $sim mcm $mcm trigger clear console $triginfo(match)
    simstop
}

proc wait_continue { str { mcm 0 } { sim mysim } } {

    $sim mcm $mcm trigger set console "$str" "wait_continue_trigger $str $sim $mcm"
}

#Assumes objectdump is an nm output file
proc get_function_entry { objectdumpfile symbolname } {
    if { ! [file exists $objectdumpfile] } {
        puts "ERROR: unable to file dump file $objectdumpfile"
        return -1
    }

    set fid [open $objectdumpfile r]
#skip first three lines
    gets $fid line
    gets $fid line
    gets $fid line
    while { ![eof $fid] } {
	# Read a line from the file and analyse it.
	gets $fid line
	#hardcoded based on format of unix_64.map
	set address [string range $line 0 7]
	set name [string range $line 11 119]
	set name [string trim $name]
        if {[string equal $name $symbolname]} {
	     return "0x$address"
	}
    }
    
    puts "ERROR: get_function_entry unable to find symbol $symbolname"
}

array unset cycle_start
array unset instr_start
array unset total_cycles
array unset total_instr
array unset entrances
array unset exits

proc profile_callback { sim symbolname args } {
    global cycle_start
    global instr_start
    global total_cycles
    global total_instr
    global entrances
    global exits

#    puts "profile entry!\n"

    set myargs [lindex $args 0]
    set cpu [lindex $myargs 5]
    set thread [lindex $myargs 7]

    set osinfostring [$sim cpu $cpu thread $thread osinfo get]
    set swtid [lindex $osinfostring 3]

    if {![info exists entrances($swtid,$symbolname)]} {
	set entrances($swtid,$symbolname) 0
    }
    
    if {![info exists exits($swtid,$symbolname)]} {
	set exits($swtid,$symbolname) 0
    }

    set entrances($swtid,$symbolname) [expr {$entrances($swtid,$symbolname) + 1}]
    set cycle_start($swtid,$symbolname)  [$sim cpu $cpu thread $thread display cycles]
    set instr_start($swtid,$symbolname) [$sim cpu $cpu thread $thread display instruction_count]
    set lr [$sim cpu $cpu thread $thread display spr lr]
    $sim trigger set pc $lr "profile_exit_callback $sim $symbolname"
}

proc profile_exit_callback { sim symbolname args } {
    global cycle_start
    global instr_start
    global total_cycles
    global total_instr
    global entrances
    global exits

#    puts "profile exit!\n"

    set myargs [lindex $args 0]
    set cpu [lindex $myargs 5]
    set thread [lindex $myargs 7]    

    set osinfostring [$sim cpu $cpu thread $thread osinfo get]
    set swtid [lindex $osinfostring 3]

    set exits($swtid,$symbolname) [expr {$exits($swtid,$symbolname) + 1}]
    set cur_cycle_count [$sim cpu $cpu thread $thread display cycles]
    set cur_instr_count [$sim cpu $cpu thread $thread display instruction_count]

    if {![info exists total_cycles($swtid,$symbolname)]} {
	set total_cycles($swtid,$symbolname) 0
    }
    
    if {![info exists total_instr($swtid,$symbolname)]} {
 	set total_instr($swtid,$symbolname) 0
    }
    
    set instr_difference [expr {$cur_instr_count - $instr_start($swtid,$symbolname)}] 
    set cycle_difference [expr {$cur_cycle_count - $cycle_start($swtid,$symbolname)}]

    if { [string equal $entrances($swtid,$symbolname) $exits($swtid,$symbolname)] } {
	set total_instr($swtid,$symbolname) [expr {$total_instr($swtid,$symbolname) + $instr_difference}]
	set total_cycles($swtid,$symbolname) [expr {$total_cycles($swtid,$symbolname) + $cycle_difference}]
    } else {
	puts "Uh oh, entr $entrances($swtid,$symbolname) != exts $exits($swtid,$symbolname)"
    }

#    dump_profile_info $symbolname
}

#Initiates profiling for a given function name. Requires nm output for binary, whose filename is
#passed in the first parameter
proc profile_function { objectdumpfile symbolname } {
    set entrypoint [get_function_entry $objectdumpfile $symbolname]
    mysim trigger set pc $entrypoint "profile_callback mysim $symbolname"
}

proc dump_profile_info {symbolname} {
    global cycle_start
    global instr_start
    global total_cycles
    global total_instr
    global entrances
    global exits
    set mytotalcycles 0
    set mytotalinstr 0

    foreach i [array names total_cycles] {
	if { ![string equal $entrances($i) $exits($i)] } {
	    puts "Uh oh, entrances != exits, entrances = $entrances($i) $exits($i)"
	}

	if { ![string equal [string first $symbolname $i] -1] } {
	    set mytotalcycles [expr {$mytotalcycles + $total_cycles($i)}]
	    set mytotalinstr [expr {$mytotalinstr + $total_instr($i)}]
	}
    }

    puts "$symbolname:$mytotalcycles:$mytotalinstr"
}

